@@ -45,6 +45,12 @@
  *   in ai_flags?
  */
 
+/*
+ * Mingw64 has its own implementation of getaddrinfo, mingw32 no
+ */
+#ifndef __MINGW64__
+ 
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif 
@@ -85,7 +91,7 @@ static const char rcsid[] _U_ =
 #ifdef NEED_ADDRINFO_H
 #include "addrinfo.h"
 #ifdef WIN32
-#include "IP6_misc.h"
+#include "ip6_misc.h"
 #endif
 #endif
 
@@ -1118,3 +1124,6 @@ find_afd(af)
 	}
 	return NULL;
 }
+
+
+#endif /*__MING64__*/
