@@ -10,13 +10,27 @@ extern "C" {
 #endif
 
 /*
- * Functions used by dlpisubs.c.
+ * Private data for capturing on DLPI devices.
+ */
+struct pcap_dlpi {
+#ifdef HAVE_LIBDLPI
+	dlpi_handle_t dlpi_hd;
+#endif /* HAVE_LIBDLPI */
+#ifdef DL_HP_RAWDLS
+	int send_fd;
+#endif /* DL_HP_RAWDLS */
+
+	struct pcap_stat stat;
+};
+
+/*
+ * Functions defined by dlpisubs.c.
  */
 int pcap_stats_dlpi(pcap_t *, struct pcap_stat *);
 int pcap_process_pkts(pcap_t *, pcap_handler, u_char *, int, u_char *, int);
 int pcap_process_mactype(pcap_t *, u_int);
 #ifdef HAVE_SYS_BUFMOD_H
-int pcap_conf_bufmod(pcap_t *, int, int);
+int pcap_conf_bufmod(pcap_t *, int);
 #endif
 int pcap_alloc_databuf(pcap_t *);
 int strioctl(int, int, int, char *);
