@@ -50,6 +50,13 @@ static int septel_setfilter(pcap_t *p, struct bpf_program *fp);
 static int septel_stats(pcap_t *p, struct pcap_stat *ps);
 static int septel_setnonblock(pcap_t *p, int nonblock, char *errbuf);
 
+/*
+ * Private data for capturing on Septel devices.
+ */
+struct pcap_septel {
+	struct pcap_stat stat;
+}
+
 /*
  *  Read at most max_packets from the capture queue and call the callback
  *  for each of them. Returns the number of packets handled, -1 if an
@@ -57,6 +64,7 @@ static int septel_setnonblock(pcap_t *p, int nonblock, char *errbuf);
  */
 static int septel_read(pcap_t *p, int cnt, pcap_handler callback, u_char *user) {
 
+  struct pcap_septel *ps = p->priv;
   HDR *h;
   MSG *m;
   int processed = 0 ;
@@ -154,7 +162,7 @@ loop:
         pcap_header.len = packet_len;
 
         /* Count the packet. */
-        p->md.stat.ps_recv++;
+        ps->stat.ps_recv++;
 
         /* Call the user supplied callback function */
         callback(user, &pcap_header, dp);
@@ -229,7 +237,7 @@ pcap_t *septel_create(const char *device, char *ebuf, int *is_ours) {
 	/* OK, it's probably ours. */
 	*is_ours = 1;
 
-	p = pcap_create_common(device, ebuf);
+	p = pcap_create_common(device, ebuf, sizeof (struct pcap_septel));
 	if (p == NULL)
 		return NULL;
 
@@ -238,10 +246,11 @@ pcap_t *septel_create(const char *device, char *ebuf, int *is_ours) {
 }
 
 static int septel_stats(pcap_t *p, struct pcap_stat *ps) {
-  /*p->md.stat.ps_recv = 0;*/
-  /*p->md.stat.ps_drop = 0;*/
+  struct pcap_septel *handlep = p->priv;
+  /*handlep->stat.ps_recv = 0;*/
+  /*handlep->stat.ps_drop = 0;*/
   
-  *ps = p->md.stat;
+  *ps = handlep->stat;
  
   return 0;
 }
@@ -282,8 +291,6 @@ static int septel_setfilter(pcap_t *p, struct bpf_program *fp) {
     return -1;
   }
 
-  p->md.use_bpf = 0;
-
   return (0);
 }
 
@@ -291,5 +298,6 @@ static int septel_setfilter(pcap_t *p, struct bpf_program *fp) {
 static int
 septel_setnonblock(pcap_t *p, int nonblock, char *errbuf)
 {
-  return (0);
+  fprintf(errbuf, PCAP_ERRBUF_SIZE, "Non-blocking mode not supported on Septel devices");
+  return (-1);
 }
